package com.gic.finance.service.outer.impl;

import java.util.Date;

import com.gic.finance.util.cashwithdrawal.StateHandler;
import org.apache.commons.lang.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.gic.api.base.commons.Page;
import com.gic.api.base.commons.ServiceResponse;
import com.gic.commons.util.EntityUtil;
import com.gic.commons.util.PageHelperUtils;
import com.gic.enterprise.error.ErrorCode;
import com.gic.enterprise.utils.AsyncCallbackUtils;
import com.gic.enterprise.utils.CreateRandomUtils;
import com.gic.enterprise.utils.valid.ValidParamsUtils;
import com.gic.finance.constant.WithdrawalApplyTypeEnum;
import com.gic.finance.constant.WithdrawalStatusEnum;
import com.gic.finance.dto.CashWithdrawalDTO;
import com.gic.finance.dto.OperationUserInfoDTO;
import com.gic.finance.entity.TabCashWithdrawal;
import com.gic.finance.entity.TabPayAccount;
import com.gic.finance.qo.CashWithdrawalQueryListQO;
import com.gic.finance.service.CashWithdrawalApiService;
import com.gic.finance.service.CashWithdrawalService;
import com.gic.finance.service.PayAccountService;
/**
 * 提现
 * @ClassName: CashWithdrawalApiServiceImpl 
 * @Description:  
 * @author guojuxing 
 * @date 2020/9/11 11:37 AM 
 */
@Service("cashWithdrawalApiService")
public class CashWithdrawalApiServiceImpl implements CashWithdrawalApiService{
    @Autowired
    private CashWithdrawalService cashWithdrawalService;
    @Autowired
    private PayAccountService payAccountService;

    @Override
    public ServiceResponse<String> cashWithdrawalOfServiceProvider(CashWithdrawalDTO dto) {
        dto.setApplyType(WithdrawalApplyTypeEnum.SERVICE_PROVIDER.getCode());
        return StateHandler.cashWithdrawal(WithdrawalStatusEnum.NEW, dto);
    }

    @Override
    public ServiceResponse<String> cashWithdrawalOfSupplier(CashWithdrawalDTO dto) {
        dto.setApplyType(WithdrawalApplyTypeEnum.SUPPLIER.getCode());
        return StateHandler.cashWithdrawal(WithdrawalStatusEnum.NEW, dto);
    }

    @Override
    public ServiceResponse<Void> expressMail(String serialNumber, String expressMailName, String expressMailNumber) {
        TabCashWithdrawal record = cashWithdrawalService.getBySerialNumber(serialNumber);
        if (record == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "提现流水号错误，查无数据");
        }
        if (StringUtils.isBlank(expressMailName)) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "物流公司不能为空");
        }
        if (StringUtils.isBlank(expressMailNumber)) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "物流单号不能为空");
        }
        record.setExpressMailName(expressMailName);
        record.setExpressMailNumber(expressMailNumber);
        cashWithdrawalService.update(record);
        return ServiceResponse.success();
    }

    @Override
    public ServiceResponse<Void> approval(Integer id, OperationUserInfoDTO dto) {
        TabCashWithdrawal record = cashWithdrawalService.getById(id);
        if (record == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "参数有误，无此记录");
        }
        // 操作人相关信息
        getOperationUserInfo(record, dto);
        return StateHandler.approval(WithdrawalStatusEnum.getEnumByCode(record.getCashWithdrawalStatus()), record);
    }

    @Override
    public ServiceResponse<CashWithdrawalDTO> getDetail(Integer id) {
        TabCashWithdrawal record = cashWithdrawalService.getById(id);
        if (record == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "参数有误，无此记录");
        }
        return ServiceResponse.success(EntityUtil.changeEntityNew(CashWithdrawalDTO.class, record));
    }

    @Override
    public ServiceResponse<Void> reject(Integer id, String rejectReason, OperationUserInfoDTO dto) {
        TabCashWithdrawal record = cashWithdrawalService.getById(id);
        if (record == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "参数有误，无此记录");
        }
        // 操作人相关信息
        getOperationUserInfo(record, dto);
        record.setRejectReason(rejectReason);
        return StateHandler.reject(WithdrawalStatusEnum.getEnumByCode(record.getCashWithdrawalStatus()), record);
    }

    @Override
    public ServiceResponse<Void> pay(Integer id, Integer payId, String bankSerialNumber) {
        TabCashWithdrawal record = cashWithdrawalService.getById(id);
        if (record == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "参数有误，无此申请单记录");
        }
        record.setBankSerialNumber(bankSerialNumber);
        record.setPayId(payId);
        return StateHandler.complete(WithdrawalStatusEnum.getEnumByCode(record.getCashWithdrawalStatus()), record);
    }

    @Override
    public ServiceResponse<Page<CashWithdrawalDTO>> listCashWithdrawalPage(CashWithdrawalQueryListQO params) {
        com.github.pagehelper.Page page = cashWithdrawalService.pageCashWithdrawal(params);
        Page<CashWithdrawalDTO> resultPage = PageHelperUtils.changePageHelperToCurrentPage(page, CashWithdrawalDTO.class);
        return ServiceResponse.success(resultPage);
    }

    @Override
    public ServiceResponse<Void> cancel(String serialNumber) {
        TabCashWithdrawal record = cashWithdrawalService.getBySerialNumber(serialNumber);
        if (record == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "提现流水号错误，查无数据");
        }
        return StateHandler.cancel(WithdrawalStatusEnum.getEnumByCode(record.getCashWithdrawalStatus()), record);
    }

    private void getOperationUserInfo(TabCashWithdrawal record, OperationUserInfoDTO userInfo) {
        record.setOperatorId(userInfo.getId().toString());
        record.setOperatorName(userInfo.getRealName());
        record.setOperatorPhone(userInfo.getUserMobile());
        record.setOperatorTime(new Date());
    }

}
