package com.gic.finance.service.outer.impl;

import java.util.Date;
import java.util.List;

import com.gic.auth.dto.UserDTO;
import com.gic.auth.service.UserApiService;
import com.gic.marketing.process.api.service.sms.SmsSendApiService;
import com.gic.open.api.dto.ApplicationDTO;
import com.gic.open.api.dto.OrderDTO;
import com.gic.open.api.service.ApplicationApiService;
import com.gic.open.api.service.OrderApiService;
import org.apache.commons.lang.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.alibaba.fastjson.JSONObject;
import com.gic.api.base.commons.Page;
import com.gic.api.base.commons.ServiceResponse;
import com.gic.commons.util.EntityUtil;
import com.gic.commons.util.PageHelperUtils;
import com.gic.enterprise.error.ErrorCode;
import com.gic.enterprise.utils.AsyncCallbackUtils;
import com.gic.enterprise.utils.CreateRandomUtils;
import com.gic.finance.constant.OrderTypeEnum;
import com.gic.finance.constant.TransferAccountApprovalStatusEnum;
import com.gic.finance.dto.OperationUserInfoDTO;
import com.gic.finance.dto.TransferAccountsApprovalDTO;
import com.gic.finance.entity.TabTransferAccountsApproval;
import com.gic.finance.qo.TransferListQueryQO;
import com.gic.finance.service.TransferAccountsApprovalApiService;
import com.gic.finance.service.TransferAccountsApprovalService;
import com.gic.store.utils.valid.ValidUtil;

/**
 * 转账审批
 * @author $user$
 * @ClassName: $ClassName$
 * @Description: $desc$
 * @date $date$ $time$
 */
@Service("transferAccountsApprovalApiService")
public class TransferAccountsApprovalApiServiceImpl implements TransferAccountsApprovalApiService {
    private static final Logger LOGGER = LoggerFactory.getLogger(TransferAccountsApprovalApiServiceImpl.class);
    private final static String SERVICE_NAME = "转账审批";
    @Autowired
    private TransferAccountsApprovalService transferAccountsApprovalService;
    @Autowired
    private OrderApiService orderApiService;
    @Autowired
    private ApplicationApiService applicationApiService;
    @Autowired
    private SmsSendApiService smsSendApiService;
    @Autowired
    private UserApiService userApiService;

    /**
     * 发起审批
     * @Title: initiateApproval 
     * @Description:
      * @author guojuxing
     * @param transferAccountsApprovalDTO 
     * @return  com.gic.api.base.commons.ServiceResponse<java.lang.Void> 
      */
    private ServiceResponse<Void> initiateApproval(TransferAccountsApprovalDTO transferAccountsApprovalDTO) {
        //valid param
        ServiceResponse paramValid = ValidUtil.allCheckValidate(transferAccountsApprovalDTO,
                TransferAccountsApprovalDTO.InitiateApprovalValid.class);
        if (!paramValid.isSuccess()) {
            return paramValid;
        }
        transferAccountsApprovalDTO.setCreateTime(new Date());
        transferAccountsApprovalDTO.setUpdateTime(new Date());
        transferAccountsApprovalDTO.setApprovalStatus(TransferAccountApprovalStatusEnum.TO_BE_APPROVED.getCode());
        if (StringUtils.isBlank(transferAccountsApprovalDTO.getEnterpriseName())) {
            transferAccountsApprovalDTO.setEnterpriseName("暂无商户名称");
        }
        if (StringUtils.isBlank(transferAccountsApprovalDTO.getCompanyName())) {
            transferAccountsApprovalDTO.setCompanyName("暂无公司名称");
        }
        if (StringUtils.isBlank(transferAccountsApprovalDTO.getLogo())) {
            transferAccountsApprovalDTO.setLogo("暂无公司logo");
        }
        if (StringUtils.isBlank(transferAccountsApprovalDTO.getInitiatorName())) {
            transferAccountsApprovalDTO.setInitiatorName("暂无发起方名称");
        }
        //审批流水号生成
        String approvalSerialNumber = CreateRandomUtils.createSerialNumber();
        LOGGER.info("转账审批-发起审批-审批流水号：{}", approvalSerialNumber);
        transferAccountsApprovalDTO.setApprovalNumber(approvalSerialNumber);

        transferAccountsApprovalService.initiateApproval(transferAccountsApprovalDTO);

        return ServiceResponse.success();
    }

    @Override
    public ServiceResponse<Void> rechargeInitiateApproval(TransferAccountsApprovalDTO transferAccountsApprovalDTO) {
        transferAccountsApprovalDTO.setOrderType(OrderTypeEnum.RECHARGE.getCode());
        return initiateApproval(transferAccountsApprovalDTO);
    }

    @Override
    public ServiceResponse<Void> smsInitiateApproval(TransferAccountsApprovalDTO transferAccountsApprovalDTO) {
        transferAccountsApprovalDTO.setOrderType(OrderTypeEnum.SHORT_MESSAGE_PACKAGE.getCode());
        return initiateApproval(transferAccountsApprovalDTO);
    }

    @Override
    public ServiceResponse<Void> serviceInitiateApproval(TransferAccountsApprovalDTO transferAccountsApprovalDTO) {
        transferAccountsApprovalDTO.setOrderType(OrderTypeEnum.SERVICE.getCode());
        return initiateApproval(transferAccountsApprovalDTO);
    }

    @Override
    public ServiceResponse<Void> extensionPackageInitiateApproval(TransferAccountsApprovalDTO transferAccountsApprovalDTO) {
        transferAccountsApprovalDTO.setOrderType(OrderTypeEnum.EXTENSION_PACKAGE.getCode());
        return initiateApproval(transferAccountsApprovalDTO);
    }

    @Override
    public ServiceResponse<Void> packageInitiateApproval(TransferAccountsApprovalDTO transferAccountsApprovalDTO) {
        transferAccountsApprovalDTO.setOrderType(OrderTypeEnum.PACKAGE.getCode());
        return initiateApproval(transferAccountsApprovalDTO);
    }

    @Override
    public ServiceResponse<TransferAccountsApprovalDTO> getById(Integer transferApprovalId) {
        LOGGER.info(SERVICE_NAME + "-getById:{}", transferApprovalId);
        TabTransferAccountsApproval tab = transferAccountsApprovalService.getById(transferApprovalId);
        if (tab == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "转账审批主键有误");
        }
        return ServiceResponse.success(EntityUtil.changeEntityNew(TransferAccountsApprovalDTO.class, tab));
    }

    @Override
    public ServiceResponse<TransferAccountsApprovalDTO> getByOrderNumber(String orderNumber) {
        TabTransferAccountsApproval tab = transferAccountsApprovalService.getByOrderNumber(orderNumber);
        return ServiceResponse.success(EntityUtil.changeEntityNew(TransferAccountsApprovalDTO.class, tab));
    }

    @Override
    public ServiceResponse<Void> agreeApproval(Integer transferApprovalId, Double accountAmount, OperationUserInfoDTO userInfoDTO) {
        TabTransferAccountsApproval tab = transferAccountsApprovalService.getById(transferApprovalId);
        if (tab == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "转账审批主键有误");
        }
        if (accountAmount == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "到账金额不能为空");
        }

        if (tab.getPlannedAmount() < accountAmount) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "到账金额不能超过计划金额");
        }
        TransferAccountsApprovalDTO dto = new TransferAccountsApprovalDTO();
        dto.setTransferApprovalId(transferApprovalId);
        dto.setAccountAmount(accountAmount);
        dto.setBadAmount(tab.getPlannedAmount() - accountAmount);
        dto.setApprovalStatus(TransferAccountApprovalStatusEnum.AGREE.getCode());

        // 审批人等信息获取插入
        operationUserInfo(dto, userInfoDTO);

        //回调所需参数
        tab.setAccountAmount(accountAmount);
        tab.setBadAmount(tab.getPlannedAmount() - accountAmount);
        tab.setApprovalStatus(TransferAccountApprovalStatusEnum.AGREE.getCode());
        tab.setApprovalTime(new Date());

        callBack(tab);
        transferAccountsApprovalService.updateTransferAccountsApproval(dto);

        //如果是商户基础服务，则进行短信发送给超级管理员
        if (tab.getOrderType().intValue() == OrderTypeEnum.SERVICE.getCode()) {
            //服务类型
            String orderNum = tab.getOrderNumber();
            ServiceResponse<OrderDTO> orderResult = orderApiService.getOrderForFinance(orderNum);
            if (!orderResult.isSuccess()) {
                LOGGER.warn("财务审批通过-服务订单信息结果:{}, {}", orderResult.getCode(), orderResult.getMessage());
            }
            OrderDTO orderDTO = orderResult.getResult();
            Long applicationId = orderDTO.getApplicationId();
            ServiceResponse<ApplicationDTO> applicationResult = applicationApiService.getApplicationSimpleByApplicationId(applicationId);
            if (!applicationResult.isSuccess()) {
                LOGGER.warn("财务审批通过-应用信息结果:{}, {}", applicationResult.getCode(), applicationResult.getMessage());
            }
            String gicModuleCode = "gic";
            if (gicModuleCode.equals(applicationResult.getResult().getModuleCode())) {
                //是基础服务
                ServiceResponse<UserDTO> userResult = userApiService.getUserByEnterpriseId(tab.getEnterpriseId());
                UserDTO userDTO = userResult.getResult();
                Integer relationOrderType = orderDTO.getOrderType();
                String operationName = "开通";
                switch (relationOrderType) {
                    case 1:
                        operationName = "开通";
                        break;
                    case 2:
                        operationName = "续费";
                        break;
                    case 3:
                        operationName = "升级";
                        break;
                    case 4:
                        operationName = "扩容";
                        break;
                    default:
                }
                String[] smsArr = new String[]{tab.getEnterpriseName(), operationName};
                if (userResult.isSuccess()) {
                    ServiceResponse<Void> smsSendResult = smsSendApiService.sendPlatformSms("GICJFZX004",
                            tab.getEnterpriseId(),
                            userDTO.getPhoneAreaCode(), userDTO.getPhoneNumber(), smsArr);
                    if (!smsSendResult.isSuccess()) {
                        LOGGER.warn(smsSendResult.getMessage());
                    }
                }
            } else {
                LOGGER.warn("财务审批通过-不是基础服务");
            }
        }
        return ServiceResponse.success();
    }

    @Override
    public ServiceResponse<Void> rejectApproval(Integer transferApprovalId, OperationUserInfoDTO userInfoDTO) {
        TabTransferAccountsApproval tab = transferAccountsApprovalService.getById(transferApprovalId);
        if (tab == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "转账审批主键有误");
        }
        TransferAccountsApprovalDTO dto = new TransferAccountsApprovalDTO();
        dto.setTransferApprovalId(transferApprovalId);
        dto.setApprovalStatus(TransferAccountApprovalStatusEnum.REJECT.getCode());
        // 审批人等信息获取插入
        operationUserInfo(dto, userInfoDTO);

        //回调所需参数
        tab.setApprovalStatus(TransferAccountApprovalStatusEnum.REJECT.getCode());
        tab.setApprovalTime(new Date());
        callBack(tab);
        transferAccountsApprovalService.updateTransferAccountsApproval(dto);
        return ServiceResponse.success();
    }

    @Override
    public ServiceResponse<Void> cancelApproval(String orderSerialNumber) {
        TabTransferAccountsApproval tab = transferAccountsApprovalService.getByOrderNumber(orderSerialNumber);
        if (tab == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "订单流水号有误");
        }
        if (tab.getApprovalStatus().intValue() != TransferAccountApprovalStatusEnum.TO_BE_APPROVED.getCode()) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "此订单已审核，不能取消");
        }
        updateCancelApproval(tab.getTransferApprovalId());
        return ServiceResponse.success();
    }

    @Override
    public ServiceResponse<Void> cancelApproval(Integer transferApprovalId) {
        TabTransferAccountsApproval tab = transferAccountsApprovalService.getById(transferApprovalId);
        if (tab == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "转账审批主键有误");
        }
        if (tab.getApprovalStatus().intValue() != TransferAccountApprovalStatusEnum.TO_BE_APPROVED.getCode()) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "此订单已审核，不能取消");
        }
        updateCancelApproval(transferApprovalId);
        return ServiceResponse.success();
    }

    @Override
    public ServiceResponse<Page<TransferAccountsApprovalDTO>> listTransferAccountsApproval(TransferListQueryQO params) {
        com.github.pagehelper.Page page = transferAccountsApprovalService.listTransferAccountsApproval(params);
        Page<TransferAccountsApprovalDTO> resultPage = PageHelperUtils.changePageHelperToCurrentPage(page, TransferAccountsApprovalDTO.class);
        return ServiceResponse.success(resultPage);
    }

    @Override
    public ServiceResponse<List<TransferAccountsApprovalDTO>> listByOrderSerialNumber(List<String> orderSerialNumberList) {
        List<TabTransferAccountsApproval> list = transferAccountsApprovalService.listByOrderSerialNumber(orderSerialNumberList);
        return ServiceResponse.success(EntityUtil.changeEntityListNew(TransferAccountsApprovalDTO.class, list));
    }

    /**
     * 状态改为取消
     * @Title: updateCancelApproval 
     * @Description:
      * @author guojuxing
     * @param transferApprovalId 
     * @return  void 
      */
    private void updateCancelApproval(Integer transferApprovalId) {
        TransferAccountsApprovalDTO dto = new TransferAccountsApprovalDTO();
        dto.setTransferApprovalId(transferApprovalId);
        dto.setApprovalStatus(TransferAccountApprovalStatusEnum.CANCEL.getCode());
        transferAccountsApprovalService.updateTransferAccountsApproval(dto);
    }

    private void callBack(TabTransferAccountsApproval tab) {
        LOGGER.info("转账审批回调：{}" ,JSONObject.toJSONString(tab));
        int orderType = tab.getOrderType().intValue();
        boolean isBilling = orderType == OrderTypeEnum.RECHARGE.getCode() ||
                orderType == OrderTypeEnum.SHORT_MESSAGE_PACKAGE.getCode();

        String interfaceUrl;
        String method;
        if (isBilling) {
            interfaceUrl = "com.gic.enterprise.service.BillingPayInfoApiService";
            method = "offlineCallBack";
        } else {
            //服务类型
            interfaceUrl = "com.gic.open.api.service.market.PayCallbackApiService";
            method = "payCallbackTransfer";
        }

        //回调
        AsyncCallbackUtils.callBack(tab, interfaceUrl, method);
    }

    private void operationUserInfo(TransferAccountsApprovalDTO dto, OperationUserInfoDTO userInfoDTO) {
        dto.setApprovalId(userInfoDTO.getId().toString());
        dto.setApprovalName(userInfoDTO.getRealName());
        dto.setApprovalPhone(userInfoDTO.getUserMobile());
        dto.setApprovalTime(new Date());
    }
}
